﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Web.Http;
using Newtonsoft.Json.Linq;

namespace QuikESignTransport.PassThru.Controllers
{
    public class SignController : ApiController
    {
      
        [HttpPost]
        [Route("sign/passthru")]
        public async Task<JObject> PassThru(HttpRequestMessage request)
        {
            try
            {
                //1. Read posted data as a raw JSON string. 
                //   It is utterly important that this string is not manipulated for a successful transaction at Docusign
                var jsonString = await request.Content.ReadAsStringAsync();

                //2. Generate a token at Quik! in order to consume the REST service and generate the envelope
                var token = GenerateQuikToken();

                //3. Invoke the rest service and generate the envelope at Docusign
                //   envelopeResponse variable contains the Docusign response after genreating an envelope, 
                //   including the envelope ID.
                var envelopeResponse = GenerateDocusignEnvelope(jsonString, token);

                //Return the JSON object version of the envelopeResponse JSON string 
                //so that the Quik! Forms Engine can read the result and notify the response to the user

                return JObject.Parse(envelopeResponse);
            }
            catch (Exception e)
            {
                 //returning error. Handle error as needed
                throw new HttpResponseException(new HttpResponseMessage(HttpStatusCode.InternalServerError)
                {
                    Content = new StringContent("An error occurred, please try again or contact the administrator."),
                    ReasonPhrase = e.Message
                });
            }
            
        }

        private string GenerateQuikToken()
        {
            var body = String.Format("grant_type=password&username={0}&password={1}", GetUsername(), GetPassword());
            var response = DoRequest(GetQuikOAuthTokenEndpoint(), body, null);
            return JObject.Parse(response)["access_token"].ToString();
        }

        private string GenerateDocusignEnvelope(string body, string quikToken)
        {
            var headers = new List<KeyValuePair<string,string>>()
            {
                new KeyValuePair<string, string>("Authorization", string.Format("BEARER {0}", quikToken))
            };

            return DoRequest(GetQuikDocusignPassThruEndpoint(), body, headers);
        }

        private string GetUsername()
        {
            return ConfigurationManager.AppSettings["QUIK_USERNAME"];
        }

        private string GetPassword()
        {
            return ConfigurationManager.AppSettings["QUIK_PASSWORD"];
        }

        private string GetQuikOAuthTokenEndpoint()
        {
            return ConfigurationManager.AppSettings["QUIK_OAUTHTOKEN_ENDPOINT"];
        }

        private string GetQuikDocusignPassThruEndpoint()
        {
            return ConfigurationManager.AppSettings["QUIK_DOCUSIGN_PASSTHRU_ENDPOINT"];
        }


        private string DoRequest(string url, string body, List<KeyValuePair<string, string>> headers)
        {
            var request = HttpWebRequest.Create(url);
            request.ContentType = "application/json";
            request.Method = "POST";

            if (headers != null)
            {
                foreach (var header in headers)
                {
                    request.Headers.Add(header.Key, header.Value);
                }
            }

            var streamBody = Encoding.UTF8.GetBytes(body);
            request.ContentLength = streamBody.Length;
            using (var requestDataStream = request.GetRequestStream())
            {
                requestDataStream.Write(streamBody, 0, streamBody.Length);
                using (var response = request.GetResponse())
                {
                    var responseDataStream = response.GetResponseStream();
                    var responseString = new StreamReader(responseDataStream).ReadToEnd();
                    return responseString;
                }
            }
        }

    }

}
